@@ -41,7 +41,7 @@ __all__ = [
 ]
 
 """
-Below minimum version is required to access the "read_idxs" argument in 
+Below minimum version is required to access the "read_idxs" argument in
 dali.fn.readers.nemo_asr
 """
 __DALI_MINIMUM_VERSION__ = "1.4"
@@ -176,6 +176,8 @@ class _AudioTextDALIDataset(Iterator):
                 f"{self} received an unexpected device argument {device}. Supported values are: 'cpu', 'gpu'"
             )
 
+        device_id = device_id if device == 'gpu' else None
+
         self.batch_size = batch_size  # Used by NeMo
 
         self.device = device
@@ -220,7 +222,7 @@ class _AudioTextDALIDataset(Iterator):
             self.window_stride_sec = params['window_stride'] if 'window_stride' in params else 0.01
             self.sample_rate = params['sample_rate'] if 'sample_rate' in params else sample_rate
             self.window_size = int(self.window_size_sec * self.sample_rate)
-            self.window_stride = int(self.window_size_sec * self.sample_rate)
+            self.window_stride = int(self.window_stride_sec * self.sample_rate)
 
             normalize = params['normalize'] if 'normalize' in params else 'per_feature'
             if normalize == 'per_feature':  # Each freq channel independently
@@ -234,28 +236,30 @@ class _AudioTextDALIDataset(Iterator):
                 )
 
             self.window = None
-            window_name = params['window'] if 'window' in params else None
+            window_name = params['window'] if 'window' in params else 'hann'
             torch_windows = {
+                'hann': torch.hann_window,
                 'hamming': torch.hamming_window,
                 'blackman': torch.blackman_window,
                 'bartlett': torch.bartlett_window,
+                'none': None,
             }
-            if window_name is None or window_name == 'hann':
-                self.window = None  # Hann is DALI's default
-            elif window_name == 'ones':
-                self.window = torch.ones(self.window_size)
+
+            if window_name == 'ones':
+                window_tensor = torch.ones(self.window_size)
             else:
                 try:
                     window_fn = torch_windows.get(window_name, None)
-                    self.window = window_fn(self.window_size, periodic=False)
                 except:
                     raise ValueError(
-                        f"{self} received {window_name} for the window parameter."
+                        f"{self} received '{window_name}' for the window parameter."
                         f" It must be one of: ('hann', 'ones', 'hamming', 'blackman', 'bartlett', None)."
                         f" None is equivalent to 'hann'."
                     )
+                window_tensor = window_fn(self.window_size, periodic=False) if window_fn else None
+            self.window = window_tensor.numpy().tolist() if window_tensor is not None else None
 
-            self.n_fft = params['n_fft'] if 'n_fft' in params else None  # None means default
+            self.n_fft = params['n_fft'] if 'n_fft' in params else 2 ** math.ceil(math.log2(self.window_size))
             self.n_mels = params['n_mels'] if 'n_mels' in params else 64
             self.n_mfcc = params['n_mfcc'] if 'n_mfcc' in params else 64
 
@@ -285,7 +289,9 @@ class _AudioTextDALIDataset(Iterator):
                     f"'clamp'."
                 )
 
-            self.log_zero_guard_value = params['log_zero_guard_value'] if 'log_zero_guard_value' in params else 1e-05
+            self.log_zero_guard_value = (
+                params['log_zero_guard_value'] if 'log_zero_guard_value' in params else 2 ** -24
+            )
             if isinstance(self.log_zero_guard_value, str):
                 if self.log_zero_guard_value == "tiny":
                     self.log_zero_guard_value = torch.finfo(torch.float32).tiny
@@ -303,7 +309,7 @@ class _AudioTextDALIDataset(Iterator):
                     f"{self} received {self.mag_power} for the mag_power parameter." f" It must be either 1.0 or 2.0."
                 )
 
-            self.pad_to = params['pad_to'] if 'pad_to' in params else 16
+            self.pad_to = max(params['pad_to'], 1) if 'pad_to' in params else 16
             self.pad_value = params['pad_value'] if 'pad_value' in params else 0.0
 
         with self.pipe:
@@ -343,16 +349,20 @@ class _AudioTextDALIDataset(Iterator):
             else:
                 # Additive gaussian noise (dither)
                 if self.dither > 0.0:
-                    gaussian_noise = dali.fn.normal_distribution(device=self.device)
+                    gaussian_noise = dali.fn.normal_distribution(audio)
                     audio = audio + self.dither * gaussian_noise
 
                 # Preemphasis filter
                 if self.preemph > 0.0:
-                    audio = dali.fn.preemphasis_filter(audio, preemph_coeff=self.preemph)
+                    audio = dali.fn.preemphasis_filter(audio, preemph_coeff=self.preemph, border='zero')
 
                 # Power spectrogram
                 spec = dali.fn.spectrogram(
-                    audio, nfft=self.n_fft, window_length=self.window_size, window_step=self.window_stride
+                    audio,
+                    nfft=self.n_fft,
+                    window_length=self.window_size,
+                    window_step=self.window_stride,
+                    window_fn=self.window,
                 )
 
                 if feature_type == 'mel_spectrogram' or feature_type == 'mfcc':
@@ -378,7 +388,7 @@ class _AudioTextDALIDataset(Iterator):
                 )
 
                 # Normalization
-                spec = dali.fn.normalize(spec, axes=self.normalization_axes)
+                spec = dali.fn.normalize(spec, axes=self.normalization_axes, epsilon=1e-5 ** 2, ddof=1)
 
                 # Extracting the length of the spectrogram
                 spec_len = dali.fn.slice(dali.fn.shapes(spec), 1, 1, axes=(0,))
