@@ -28,7 +28,7 @@ from nemo.collections.asr.data import audio_to_text_dataset
 from nemo.collections.asr.data.audio_to_text_dali import DALIOutputs
 from nemo.collections.asr.losses.rnnt import RNNTLoss, resolve_rnnt_default_loss_name
 from nemo.collections.asr.metrics.rnnt_wer import RNNTWER, RNNTDecoding
-from nemo.collections.asr.models.asr_model import ASRModel
+from nemo.collections.asr.models.asr_model import ASRModel, ExportableEncDecJointModel
 from nemo.collections.asr.parts.mixins import ASRModuleMixin
 from nemo.collections.asr.parts.preprocessing.perturb import process_augmentations
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
@@ -36,7 +36,7 @@ from nemo.core.neural_types import AcousticEncodedRepresentation, AudioSignal, L
 from nemo.utils import logging
 
 
-class EncDecRNNTModel(ASRModel, ASRModuleMixin):
+class EncDecRNNTModel(ASRModel, ASRModuleMixin, ExportableEncDecJointModel):
     """Base class for encoder decoder RNNT-based models."""
 
     @classmethod
@@ -206,7 +206,7 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
     @torch.no_grad()
     def transcribe(
         self, paths2audio_files: List[str], batch_size: int = 4, return_hypotheses: bool = False
-    ) -> List[str]:
+    ) -> (List[str], Optional[List['Hypothesis']]):
         """
         Uses greedy decoding to transcribe audio files. Use this method for debugging and prototyping.
 
@@ -219,18 +219,25 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
         Bigger will result in better throughput performance but would use more memory.
             return_hypotheses: (bool) Either return hypotheses or text
         With hypotheses can do some postprocessing like getting timestamp or rescoring
-        Returns:
 
-            A list of transcriptions in the same order as paths2audio_files
+        Returns:
+            A list of transcriptions in the same order as paths2audio_files. Will also return
         """
         if paths2audio_files is None or len(paths2audio_files) == 0:
             return {}
         # We will store transcriptions here
         hypotheses = []
+        all_hypotheses = []
         # Model's mode and device
         mode = self.training
         device = next(self.parameters()).device
+        dither_value = self.preprocessor.featurizer.dither
+        pad_to_value = self.preprocessor.featurizer.pad_to
+
         try:
+            self.preprocessor.featurizer.dither = 0.0
+            self.preprocessor.featurizer.pad_to = 0
+
             # Switch model to evaluation mode
             self.eval()
             # Freeze the encoder and decoder modules
@@ -253,20 +260,30 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
                     encoded, encoded_len = self.forward(
                         input_signal=test_batch[0].to(device), input_signal_length=test_batch[1].to(device)
                     )
-                    hypotheses += self.decoding.rnnt_decoder_predictions_tensor(
+                    best_hyp, all_hyp = self.decoding.rnnt_decoder_predictions_tensor(
                         encoded, encoded_len, return_hypotheses=return_hypotheses
                     )
+
+                    hypotheses += best_hyp
+                    if all_hyp is not None:
+                        all_hypotheses += all_hyp
+                    else:
+                        all_hypotheses += best_hyp
+
                     del encoded
                     del test_batch
         finally:
             # set mode back to its original value
             self.train(mode=mode)
+            self.preprocessor.featurizer.dither = dither_value
+            self.preprocessor.featurizer.pad_to = pad_to_value
+
             logging.set_verbosity(logging_level)
             if mode is True:
                 self.encoder.unfreeze()
                 self.decoder.unfreeze()
                 self.joint.unfreeze()
-        return hypotheses
+        return hypotheses, all_hypotheses
 
     def change_vocabulary(self, new_vocabulary: List[str], decoding_cfg: Optional[DictConfig] = None):
         """
@@ -612,7 +629,7 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
         del signal
 
         # During training, loss must be computed, so decoder forward is necessary
-        decoder, target_length = self.decoder(targets=transcript, target_length=transcript_len)
+        decoder, target_length, states = self.decoder(targets=transcript, target_length=transcript_len)
 
         if hasattr(self, '_trainer') and self._trainer is not None:
             log_every_n_steps = self._trainer.log_every_n_steps
@@ -684,7 +701,7 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
         # If experimental fused Joint-Loss-WER is not used
         if not self.joint.fuse_loss_wer:
             if self.compute_eval_loss:
-                decoder, target_length = self.decoder(targets=transcript, target_length=transcript_len)
+                decoder, target_length, states = self.decoder(targets=transcript, target_length=transcript_len)
                 joint = self.joint(encoder_outputs=encoded, decoder_outputs=decoder)
 
                 loss_value = self.loss(
@@ -706,7 +723,7 @@ class EncDecRNNTModel(ASRModel, ASRModuleMixin):
             compute_wer = True
 
             if self.compute_eval_loss:
-                decoded, target_len = self.decoder(targets=transcript, target_length=transcript_len)
+                decoded, target_len, states = self.decoder(targets=transcript, target_length=transcript_len)
             else:
                 decoded = None
                 target_len = transcript_len
