@@ -40,7 +40,7 @@ class TextNormalizationTestDataset:
 
     def __init__(self, input_file: str, mode: str, lang: str):
         self.lang = lang
-        insts = read_data_file(input_file)
+        insts = read_data_file(input_file, lang=lang)
 
         # Build inputs and targets
         self.directions, self.inputs, self.targets, self.classes, self.nb_spans, self.span_starts, self.span_ends = (
@@ -54,67 +54,90 @@ class TextNormalizationTestDataset:
         )
         for (classes, w_words, s_words) in insts:
             # Extract words that are not punctuations
-            (
-                processed_w_words,
-                processed_s_words,
-                processed_classes_tn,
-                processed_classes_itn,
-                processed_nb_spans_tn,
-                processed_nb_spans_itn,
-                w_span_starts,
-                w_span_ends,
-                processed_s_span_starts,
-                processed_s_span_ends,
-            ) = ([], [], [], [], 0, 0, [], [], [], [])
-            w_word_idx = 0
-            s_word_idx = 0
-            for cls, w_word, s_word in zip(classes, w_words, s_words):
-                w_span_starts.append(w_word_idx)
-                w_word_idx += len(basic_tokenize(w_word, lang=self.lang))
-                w_span_ends.append(w_word_idx)
-                processed_nb_spans_tn += 1
-                if s_word == constants.SIL_WORD:
-                    processed_classes_tn.append(constants.PUNCT_TAG)
-                    continue
-                if s_word == constants.SELF_WORD:
-                    processed_s_words.append(w_word)
-                if not s_word in constants.SPECIAL_WORDS:
-                    processed_s_words.append(s_word)
-                processed_nb_spans_itn += 1
-                processed_classes_tn.append(cls)
-                processed_classes_itn.append(cls)
-                processed_s_span_starts.append(s_word_idx)
-                s_word_idx += len(basic_tokenize(processed_s_words[-1], lang=self.lang))
-                processed_s_span_ends.append(s_word_idx)
-                processed_w_words.append(w_word)
-            # Create examples
             for direction in constants.INST_DIRECTIONS:
                 if direction == constants.INST_BACKWARD:
                     if mode == constants.TN_MODE:
                         continue
-                    input_words = processed_s_words
-                    output_words = processed_w_words
+
+                    # ITN mode
+                    (
+                        processed_w_words,
+                        processed_s_words,
+                        processed_classes,
+                        processed_nb_spans,
+                        processed_s_span_starts,
+                        processed_s_span_ends,
+                    ) = ([], [], [], 0, [], [])
+                    s_word_idx = 0
+                    for cls, w_word, s_word in zip(classes, w_words, s_words):
+                        if s_word == constants.SIL_WORD:
+                            continue
+                        elif s_word == constants.SELF_WORD:
+                            processed_s_words.append(w_word)
+                        else:
+                            processed_s_words.append(s_word)
+
+                        processed_nb_spans += 1
+                        processed_classes.append(cls)
+                        processed_s_span_starts.append(s_word_idx)
+                        s_word_idx += len(basic_tokenize(processed_s_words[-1], lang=self.lang))
+                        processed_s_span_ends.append(s_word_idx)
+                        processed_w_words.append(w_word)
+
                     self.span_starts.append(processed_s_span_starts)
                     self.span_ends.append(processed_s_span_ends)
-                    self.classes.append(processed_classes_itn)
-                    self.nb_spans.append(processed_nb_spans_itn)
-                if direction == constants.INST_FORWARD:
+                    self.classes.append(processed_classes)
+                    self.nb_spans.append(processed_nb_spans)
+                    # Basic tokenization
+                    input_words = basic_tokenize(' '.join(processed_s_words), lang)
+                    # Update self.directions, self.inputs, self.targets
+                    self.directions.append(direction)
+                    self.inputs.append(' '.join(input_words))
+                    self.targets.append(
+                        processed_w_words
+                    )  # is list of lists where inner list contains target tokens (not words)
+
+                # TN mode
+                elif direction == constants.INST_FORWARD:
                     if mode == constants.ITN_MODE:
                         continue
-                    input_words = w_words
-                    output_words = processed_s_words
+                    (
+                        processed_w_words,
+                        processed_s_words,
+                        processed_classes,
+                        processed_nb_spans,
+                        w_span_starts,
+                        w_span_ends,
+                    ) = ([], [], [], 0, [], [])
+                    w_word_idx = 0
+                    for cls, w_word, s_word in zip(classes, w_words, s_words):
+
+                        # TN forward mode
+                        if s_word in constants.SPECIAL_WORDS:
+                            processed_s_words.append(w_word)
+                        else:
+                            processed_s_words.append(s_word)
+
+                        w_span_starts.append(w_word_idx)
+                        w_word_idx += len(basic_tokenize(w_word, lang=self.lang))
+                        w_span_ends.append(w_word_idx)
+                        processed_nb_spans += 1
+                        processed_classes.append(cls)
+                        processed_w_words.append(w_word)
+
                     self.span_starts.append(w_span_starts)
                     self.span_ends.append(w_span_ends)
-                    self.classes.append(processed_classes_tn)
-                    self.nb_spans.append(processed_nb_spans_tn)
-                # Basic tokenization
-                input_words = basic_tokenize(' '.join(input_words), lang)
-                # Update self.directions, self.inputs, self.targets
-                self.directions.append(direction)
-                self.inputs.append(' '.join(input_words))
-                self.targets.append(
-                    output_words
-                )  # is list of lists where inner list contains target tokens (not words)
+                    self.classes.append(processed_classes)
+                    self.nb_spans.append(processed_nb_spans)
+                    # Basic tokenization
+                    input_words = basic_tokenize(' '.join(processed_w_words), lang)
+                    # Update self.directions, self.inputs, self.targets
+                    self.directions.append(direction)
+                    self.inputs.append(' '.join(input_words))
+                    self.targets.append(
+                        processed_s_words
+                    )  # is list of lists where inner list contains target tokens (not words)
+
         self.examples = list(
             zip(
                 self.directions,
@@ -208,9 +231,9 @@ class TextNormalizationTestDataset:
 
         if len(targets) == 0:
             return 'NA'
-
         class2stats, class2correct = defaultdict(int), defaultdict(int)
         for ix, (sent, tags) in enumerate(zip(inputs, tag_preds)):
+            assert len(inputs) == len(tag_preds)
             cur_words = [[] for _ in range(nb_spans[ix])]
             jx, span_idx = 0, 0
             cur_spans = output_spans[ix]
@@ -225,9 +248,6 @@ class TextNormalizationTestDataset:
                 if constants.SAME_TAG in tag:
                     cur_words[class_idx].append(word)
                     jx += 1
-                elif constants.PUNCT_TAG in tag:
-                    cur_words[class_idx].append(word)
-                    jx += 1
                 else:
                     jx += 1
                     tmp = cur_spans[span_idx]
@@ -242,8 +262,6 @@ class TextNormalizationTestDataset:
 
             target_token_idx = 0
             for class_idx in range(nb_spans[ix]):
-                if classes[ix][class_idx] == constants.PUNCT_TAG:
-                    continue
                 correct = TextNormalizationTestDataset.is_same(
                     " ".join(cur_words[class_idx]), targets[ix][target_token_idx], inst_directions[ix], lang
                 )
