@@ -12,8 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from time import perf_counter
-from typing import List, Optional
+import json
+import os
+from collections import defaultdict
+from typing import Dict, List, Optional, Union
 
 import nltk
 import torch
@@ -22,8 +24,13 @@ from omegaconf import DictConfig
 from pytorch_lightning import Trainer
 from transformers import AutoModelForSeq2SeqLM, AutoTokenizer, DataCollatorForSeq2Seq
 
-from nemo.collections.nlp.data.text_normalization import TextNormalizationDecoderDataset, constants
-from nemo.collections.nlp.models.duplex_text_normalization.utils import is_url
+import nemo.collections.nlp.data.text_normalization.constants as constants
+from nemo.collections.common.tokenizers.moses_tokenizers import MosesProcessor
+from nemo.collections.nlp.data.text_normalization.decoder_dataset import (
+    TarredTextNormalizationDecoderDataset,
+    TextNormalizationDecoderDataset,
+)
+from nemo.collections.nlp.models.duplex_text_normalization.utils import get_formatted_string, is_url
 from nemo.collections.nlp.models.nlp_model import NLPModel
 from nemo.core.classes.common import PretrainedModelInfo
 from nemo.utils import logging
@@ -49,9 +56,19 @@ class DuplexDecoderModel(NLPModel):
     """
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
+        # Get global rank and total number of GPU workers for IterableDataset partitioning, if applicable
+        # Global_rank and local_rank is set by LightningModule in Lightning 1.2.0
+        self.world_size = 1
+        if trainer is not None:
+            self.world_size = trainer.num_nodes * trainer.num_gpus
+
         self._tokenizer = AutoTokenizer.from_pretrained(cfg.tokenizer)
+
         super().__init__(cfg=cfg, trainer=trainer)
         self.model = AutoModelForSeq2SeqLM.from_pretrained(cfg.transformer)
+        self.max_sequence_len = cfg.get('max_seq_length', self._tokenizer.model_max_length)
+        self.mode = cfg.get('mode', 'joint')
+
         self.transformer_name = cfg.transformer
 
         # Language
@@ -64,6 +81,9 @@ class DuplexDecoderModel(NLPModel):
         if self.use_cg:
             self.setup_cgs(cfg)
 
+        # setup processor for detokenization
+        self.processor = MosesProcessor(lang_id=self.lang)
+
     # Setup covering grammars (if enabled)
     def setup_cgs(self, cfg: DictConfig):
         """
@@ -90,6 +110,11 @@ class DuplexDecoderModel(NLPModel):
         Lightning calls this inside the training loop with the data from the training dataloader
         passed in as `batch`.
         """
+        # tarred dataset contains batches, and the first dimension of size 1 added by the DataLoader
+        # (batch_size is set to 1) is redundant
+        if batch['input_ids'].ndim == 3:
+            batch = {k: v.squeeze(dim=0) for k, v in batch.items()}
+
         # Apply Transformer
         outputs = self.model(
             input_ids=batch['input_ids'],
@@ -105,12 +130,11 @@ class DuplexDecoderModel(NLPModel):
         return {'loss': train_loss, 'lr': lr}
 
     # Validation and Testing
-    def validation_step(self, batch, batch_idx):
+    def validation_step(self, batch, batch_idx, dataloader_idx=0, split="val"):
         """
         Lightning calls this inside the validation loop with the data from the validation dataloader
         passed in as `batch`.
         """
-
         # Apply Transformer
         outputs = self.model(
             input_ids=batch['input_ids'],
@@ -120,33 +144,145 @@ class DuplexDecoderModel(NLPModel):
         )
         val_loss = outputs.loss
 
-        return {'val_loss': val_loss}
+        labels_str = self._tokenizer.batch_decode(
+            torch.ones_like(batch['labels']) * ((batch['labels'] == -100) * 100) + batch['labels'],
+            skip_special_tokens=True,
+        )
+        generated_texts, _, _ = self._generate_predictions(
+            input_ids=batch['input_ids'], model_max_len=self.max_sequence_len
+        )
 
-    def validation_epoch_end(self, outputs):
+        input_centers = self._tokenizer.batch_decode(batch['input_center'], skip_special_tokens=True)
+        direction = [x[0].item() for x in batch['direction']]
+        direction_str = [constants.DIRECTIONS_ID_TO_NAME[x] for x in direction]
+        # apply post_processing
+        generated_texts = self.postprocess_output_spans(input_centers, generated_texts, direction_str)
+        results = defaultdict(int)
+        for idx, class_id in enumerate(batch['semiotic_class_id']):
+            direction = constants.TASK_ID_TO_MODE[batch['direction'][idx][0].item()]
+            class_name = self._val_id_to_class[dataloader_idx][class_id[0].item()]
+            results[f"correct_{class_name}_{direction}"] += torch.tensor(
+                labels_str[idx] == generated_texts[idx], dtype=torch.int
+            ).to(self.device)
+            results[f"total_{class_name}_{direction}"] += torch.tensor(1).to(self.device)
+
+        results[f"{split}_loss"] = val_loss
+        return dict(results)
+
+    def multi_validation_epoch_end(self, outputs: List, dataloader_idx=0, split="val"):
         """
         Called at the end of validation to aggregate outputs.
-        :param outputs: list of individual outputs of each validation step.
-        """
-        avg_loss = torch.stack([x['val_loss'] for x in outputs]).mean()
-        self.log('val_loss', avg_loss)
 
+        Args:
+            outputs: list of individual outputs of each validation step.
+        """
+        avg_loss = torch.stack([x[f'{split}_loss'] for x in outputs]).mean()
+
+        # create a dictionary to store all the results
+        results = {}
+        directions = [constants.TN_MODE, constants.ITN_MODE] if self.mode == constants.JOINT_MODE else [self.mode]
+        for class_name in self._val_class_to_id[dataloader_idx]:
+            for direction in directions:
+                results[f"correct_{class_name}_{direction}"] = 0
+                results[f"total_{class_name}_{direction}"] = 0
+
+        for key in results:
+            count = [x[key] for x in outputs if key in x]
+            count = torch.stack(count).sum() if len(count) > 0 else torch.tensor(0).to(self.device)
+            results[key] = count
+
+        all_results = defaultdict(list)
+
+        if torch.distributed.is_initialized():
+            world_size = torch.distributed.get_world_size()
+            for ind in range(world_size):
+                for key, v in results.items():
+                    all_results[key].append(torch.empty_like(v))
+            for key, v in results.items():
+                torch.distributed.all_gather(all_results[key], v)
+        else:
+            for key, v in results.items():
+                all_results[key].append(v)
+
+        if not torch.distributed.is_initialized() or torch.distributed.get_rank() == 0:
+            if split == "test":
+                val_name = self._test_names[dataloader_idx].upper()
+            else:
+                val_name = self._validation_names[dataloader_idx].upper()
+            final_results = defaultdict(int)
+            for key, v in all_results.items():
+                for _v in v:
+                    final_results[key] += _v.item()
+
+            accuracies = defaultdict(dict)
+            for key, value in final_results.items():
+                if "total_" in key:
+                    _, class_name, mode = key.split('_')
+                    correct = final_results[f"correct_{class_name}_{mode}"]
+                    if value == 0:
+                        accuracies[mode][class_name] = (0, correct, value)
+                    else:
+                        acc = round(correct / value * 100, 3)
+                        accuracies[mode][class_name] = (acc, correct, value)
+
+            for mode, values in accuracies.items():
+                report = f"Accuracy {mode.upper()} task {val_name}:\n"
+                report += '\n'.join(
+                    [
+                        get_formatted_string((class_name, f'{v[0]}% ({v[1]}/{v[2]})'), str_max_len=24)
+                        for class_name, v in values.items()
+                    ]
+                )
+                # calculate average across all classes
+                all_total = 0
+                all_correct = 0
+                for _, class_values in values.items():
+                    _, correct, total = class_values
+                    all_correct += correct
+                    all_total += total
+                all_acc = round((all_correct / all_total) * 100, 3) if all_total > 0 else 0
+                report += '\n' + get_formatted_string(
+                    ('AVG', f'{all_acc}% ({all_correct}/{all_total})'), str_max_len=24
+                )
+                logging.info(report)
+                accuracies[mode]['AVG'] = [all_acc]
+
+        self.log(f'{split}_loss', avg_loss)
+        if self.trainer.is_global_zero:
+            for mode in accuracies:
+                for class_name, values in accuracies[mode].items():
+                    self.log(f'{val_name}_{mode.upper()}_acc_{class_name.upper()}', values[0], rank_zero_only=True)
         return {
-            'val_loss': avg_loss,
+            f'{split}_loss': avg_loss,
         }
 
-    def test_step(self, batch, batch_idx):
+    def test_step(self, batch, batch_idx, dataloader_idx: int = 0):
         """
         Lightning calls this inside the test loop with the data from the test dataloader
         passed in as `batch`.
         """
-        return self.validation_step(batch, batch_idx)
+        return self.validation_step(batch, batch_idx, dataloader_idx, split="test")
 
-    def test_epoch_end(self, outputs):
+    def multi_test_epoch_end(self, outputs, dataloader_idx: int = 0):
         """
         Called at the end of test to aggregate outputs.
-        :param outputs: list of individual outputs of each test step.
+        outputs: list of individual outputs of each test step.
         """
-        return self.validation_epoch_end(outputs)
+        return self.multi_validation_epoch_end(outputs, dataloader_idx, split="test")
+
+    @torch.no_grad()
+    def _generate_predictions(self, input_ids: torch.Tensor, model_max_len: int = 512):
+        """
+        Generates predictions
+        """
+        outputs = self.model.generate(
+            input_ids, output_scores=True, return_dict_in_generate=True, max_length=model_max_len
+        )
+
+        generated_ids, sequence_toks_scores = outputs['sequences'], outputs['scores']
+        generated_texts = self._tokenizer.batch_decode(generated_ids, skip_special_tokens=True)
+
+        return generated_texts, generated_ids, sequence_toks_scores
 
     # Functions for inference
     @torch.no_grad()
@@ -181,7 +317,11 @@ class DuplexDecoderModel(NLPModel):
         extra_id_0 = constants.EXTRA_ID_0
         extra_id_1 = constants.EXTRA_ID_1
 
-        # Build all_inputs
+        """
+        Build all_inputs - extracted spans to be transformed by the decoder model
+        Inputs for TN direction have "0" prefix, while the backward, ITN direction, has prefix "1"
+        "input_centers" - List[str] - ground-truth labels for the span #TODO: rename
+        """
         input_centers, input_dirs, all_inputs = [], [], []
         for ix, sent in enumerate(sents):
             cur_inputs = []
@@ -209,9 +349,10 @@ class DuplexDecoderModel(NLPModel):
         # Apply the decoding model
         batch = tokenizer(all_inputs, padding=True, return_tensors='pt')
         input_ids = batch['input_ids'].to(self.device)
-        outputs = model.generate(input_ids, output_scores=True, return_dict_in_generate=True, max_length=model_max_len)
-        generated_ids, sequence_toks_scores = outputs['sequences'], outputs['scores']
-        generated_texts = tokenizer.batch_decode(generated_ids, skip_special_tokens=True)
+
+        generated_texts, generated_ids, sequence_toks_scores = self._generate_predictions(
+            input_ids=input_ids, model_max_len=model_max_len
+        )
 
         # Use covering grammars (if enabled)
         if self.use_cg:
@@ -256,10 +397,21 @@ class DuplexDecoderModel(NLPModel):
 
         return final_texts
 
-    def postprocess_output_spans(self, input_centers, output_spans, input_dirs):
+    def postprocess_output_spans(self, input_centers: List[str], generated_spans: List[str], input_dirs: List[str]):
+        """
+        Post processing of the generated texts
+
+        Args:
+            input_centers: Input str (no special tokens or context)
+            generated_spans: Generated spans
+            input_dirs: task direction: constants.INST_BACKWARD or constants.INST_FORWARD
+
+        Returns:
+            Processing texts
+        """
         en_greek_writtens = list(constants.EN_GREEK_TO_SPOKEN.keys())
         en_greek_spokens = list(constants.EN_GREEK_TO_SPOKEN.values())
-        for ix, (_input, _output) in enumerate(zip(input_centers, output_spans)):
+        for ix, (_input, _output) in enumerate(zip(input_centers, generated_spans)):
             if self.lang == constants.ENGLISH:
                 # Handle URL
                 if is_url(_input):
@@ -272,18 +424,18 @@ class DuplexDecoderModel(NLPModel):
                     _output = _output.replace('%', ' percent ')
                     _output = _output.replace('www', ' w w w ')
                     _output = _output.replace('ftp', ' f t p ')
-                    output_spans[ix] = ' '.join(wordninja.split(_output))
+                    generated_spans[ix] = ' '.join(wordninja.split(_output))
                     continue
                 # Greek letters
                 if _input in en_greek_writtens:
                     if input_dirs[ix] == constants.INST_FORWARD:
-                        output_spans[ix] = constants.EN_GREEK_TO_SPOKEN[_input]
+                        generated_spans[ix] = constants.EN_GREEK_TO_SPOKEN[_input]
                 if _input in en_greek_spokens:
                     if input_dirs[ix] == constants.INST_FORWARD:
-                        output_spans[ix] = _input
+                        generated_spans[ix] = _input
                     if input_dirs[ix] == constants.INST_BACKWARD:
-                        output_spans[ix] = constants.EN_SPOKEN_TO_GREEK[_input]
-        return output_spans
+                        generated_spans[ix] = constants.EN_SPOKEN_TO_GREEK[_input]
+        return generated_spans
 
     # Functions for processing data
     def setup_training_data(self, train_data_config: Optional[DictConfig]):
@@ -293,7 +445,9 @@ class DuplexDecoderModel(NLPModel):
             )
             self.train_dataset, self._train_dl = None, None
             return
-        self.train_dataset, self._train_dl = self._setup_dataloader_from_config(cfg=train_data_config, mode="train")
+        self.train_dataset, self._train_dl = self._setup_dataloader_from_config(
+            cfg=train_data_config, data_split="train"
+        )
 
     def setup_validation_data(self, val_data_config: Optional[DictConfig]):
         if not val_data_config or not val_data_config.data_path:
@@ -303,9 +457,19 @@ class DuplexDecoderModel(NLPModel):
             self.validation_dataset, self._validation_dl = None, None
             return
         self.validation_dataset, self._validation_dl = self._setup_dataloader_from_config(
-            cfg=val_data_config, mode="val"
+            cfg=val_data_config, data_split="val"
         )
 
+    def setup_multiple_validation_data(self, val_data_config: Union[DictConfig, Dict] = None):
+        if val_data_config is None:
+            val_data_config = self._cfg.validation_ds
+        return super().setup_multiple_validation_data(val_data_config)
+
+    def setup_multiple_test_data(self, test_data_config: Union[DictConfig, Dict] = None):
+        if test_data_config is None:
+            test_data_config = self._cfg.test_ds
+        return super().setup_multiple_test_data(test_data_config)
+
     def setup_test_data(self, test_data_config: Optional[DictConfig]):
         if not test_data_config or test_data_config.data_path is None:
             logging.info(
@@ -313,33 +477,85 @@ class DuplexDecoderModel(NLPModel):
             )
             self.test_dataset, self._test_dl = None, None
             return
-        self.test_dataset, self._test_dl = self._setup_dataloader_from_config(cfg=test_data_config, mode="test")
+        self.test_dataset, self._test_dl = self._setup_dataloader_from_config(cfg=test_data_config, data_split="test")
+
+    def _setup_dataloader_from_config(self, cfg: DictConfig, data_split: str):
+        logging.info(f"Creating {data_split} dataset")
+
+        shuffle = cfg["shuffle"]
+
+        if cfg.get("use_tarred_dataset", False):
+            logging.info('Tarred dataset')
+            metadata_file = cfg["tar_metadata_file"]
+            if metadata_file is None or not os.path.exists(metadata_file):
+                raise FileNotFoundError(f"Trying to use tarred dataset but could not find {metadata_file}.")
+
+            with open(metadata_file, "r") as f:
+                metadata = json.load(f)
+                num_batches = metadata["num_batches"]
+                tar_files = os.path.join(os.path.dirname(metadata_file), metadata["text_tar_filepaths"])
+            logging.info(f"Loading {tar_files}")
+
+            dataset = TarredTextNormalizationDecoderDataset(
+                text_tar_filepaths=tar_files,
+                num_batches=num_batches,
+                shuffle_n=cfg.get("tar_shuffle_n", 4 * cfg['batch_size']) if shuffle else 0,
+                shard_strategy=cfg.get("shard_strategy", "scatter"),
+                global_rank=self.global_rank,
+                world_size=self.world_size,
+            )
+
+            dl = torch.utils.data.DataLoader(
+                dataset=dataset,
+                batch_size=1,
+                sampler=None,
+                num_workers=cfg.get("num_workers", 2),
+                pin_memory=cfg.get("pin_memory", False),
+                drop_last=cfg.get("drop_last", False),
+            )
+        else:
+            input_file = cfg.data_path
+            if not os.path.exists(input_file):
+                raise ValueError(f"{input_file} not found.")
+
+            dataset = TextNormalizationDecoderDataset(
+                input_file=input_file,
+                tokenizer=self._tokenizer,
+                tokenizer_name=self.transformer_name,
+                mode=self.mode,
+                max_len=self.max_sequence_len,
+                decoder_data_augmentation=cfg.get('decoder_data_augmentation', False)
+                if data_split == "train"
+                else False,
+                lang=self.lang,
+                do_basic_tokenize=cfg.do_basic_tokenize,
+                use_cache=cfg.get('use_cache', False),
+                max_insts=cfg.get('max_insts', -1),
+                do_tokenize=True,
+            )
+
+            # create and save class names to class_ids mapping for validation
+            # (each validation set might have different classes)
+            if data_split in ['val', 'test']:
+                if not hasattr(self, "_val_class_to_id"):
+                    self._val_class_to_id = []
+                    self._val_id_to_class = []
+                self._val_class_to_id.append(dataset.label_ids_semiotic)
+                self._val_id_to_class.append({v: k for k, v in dataset.label_ids_semiotic.items()})
+
+            data_collator = DataCollatorForSeq2Seq(
+                self._tokenizer, model=self.model, label_pad_token_id=constants.LABEL_PAD_TOKEN_ID, padding=True
+            )
+            dl = torch.utils.data.DataLoader(
+                dataset=dataset,
+                batch_size=cfg.batch_size,
+                shuffle=shuffle,
+                collate_fn=data_collator,
+                num_workers=cfg.get("num_workers", 3),
+                pin_memory=cfg.get("pin_memory", False),
+                drop_last=cfg.get("drop_last", False),
+            )
 
-    def _setup_dataloader_from_config(self, cfg: DictConfig, mode: str):
-        tokenizer, model = self._tokenizer, self.model
-        start_time = perf_counter()
-        logging.info(f'Creating {mode} dataset')
-        input_file = cfg.data_path
-        dataset = TextNormalizationDecoderDataset(
-            input_file,
-            tokenizer,
-            self.transformer_name,
-            cfg.mode,
-            cfg.get('max_decoder_len', tokenizer.model_max_length),
-            cfg.get('decoder_data_augmentation', False),
-            cfg.lang,
-            cfg.do_basic_tokenize,
-            cfg.get('use_cache', False),
-            cfg.get('max_insts', -1),
-        )
-        data_collator = DataCollatorForSeq2Seq(
-            tokenizer, model=model, label_pad_token_id=constants.LABEL_PAD_TOKEN_ID,
-        )
-        dl = torch.utils.data.DataLoader(
-            dataset=dataset, batch_size=cfg.batch_size, shuffle=cfg.shuffle, collate_fn=data_collator
-        )
-        running_time = perf_counter() - start_time
-        logging.info(f'Took {running_time} seconds')
         return dataset, dl
 
     @classmethod
