@@ -21,6 +21,7 @@ from omegaconf import DictConfig
 from pytorch_lightning import Trainer
 from torch import nn
 from transformers import AutoModelForTokenClassification, AutoTokenizer, DataCollatorForTokenClassification
+from transformers.tokenization_utils_base import BatchEncoding
 
 from nemo.collections.nlp.data.text_normalization import TextNormalizationTaggerDataset, constants
 from nemo.collections.nlp.metrics.classification_report import ClassificationReport
@@ -46,16 +47,18 @@ class DuplexTaggerModel(NLPModel):
         self._tokenizer = AutoTokenizer.from_pretrained(cfg.tokenizer, add_prefix_space=True)
         super().__init__(cfg=cfg, trainer=trainer)
         self.num_labels = len(constants.ALL_TAG_LABELS)
+        self.mode = cfg.get('mode', 'joint')
+
         self.model = AutoModelForTokenClassification.from_pretrained(cfg.transformer, num_labels=self.num_labels)
         self.transformer_name = cfg.transformer
+        self.max_sequence_len = cfg.get('max_sequence_len', self._tokenizer.model_max_length)
 
         # Loss Functions
         self.loss_fct = nn.CrossEntropyLoss(ignore_index=constants.LABEL_PAD_TOKEN_ID)
 
         # setup to track metrics
-        label_ids = {l: idx for idx, l in enumerate(constants.ALL_TAG_LABELS)}
         self.classification_report = ClassificationReport(
-            self.num_labels, label_ids, mode='micro', dist_sync_on_step=True
+            self.num_labels, constants.LABEL_IDS, mode='micro', dist_sync_on_step=True
         )
 
         # Language
@@ -68,7 +71,6 @@ class DuplexTaggerModel(NLPModel):
         passed in as `batch`.
         """
         num_labels = self.num_labels
-
         # Apply Transformer
         tag_logits = self.model(batch['input_ids'], batch['attention_mask']).logits
 
@@ -129,17 +131,20 @@ class DuplexTaggerModel(NLPModel):
 
     # Functions for inference
     @torch.no_grad()
-    def _infer(self, sents: List[List[str]], inst_directions: List[str]):
+    def _infer(self, sents: List[List[str]], inst_directions: List[str], do_basic_tokenization=True):
         """ Main function for Inference
+
         Args:
             sents: A list of inputs tokenized by a basic tokenizer.
-            inst_directions: A list of str where each str indicates the direction of the corresponding instance (i.e., INST_BACKWARD for ITN or INST_FORWARD for TN).
+            inst_directions: A list of str where each str indicates the direction of the corresponding instance
+                (i.e., INST_BACKWARD for ITN or INST_FORWARD for TN).
 
         Returns:
-            all_tag_preds: A list of list where each list contains the raw tag predictions for the corresponding input.
-            nb_spans: A list of ints where each int indicates the number of semiotic spans in each input.
-            span_starts: A list of lists where each list contains the starting locations of semiotic spans in an input.
-            span_ends: A list of lists where each list contains the ending locations of semiotic spans in an input.
+            all_tag_preds: A list of list where each list contains the raw tag predictions for the corresponding input words in sents.
+            nb_spans: A list of ints where each int indicates the number of semiotic spans in input words.
+            span_starts: A list of lists where each list contains the starting locations of semiotic spans in input words.
+            span_ends: A list of lists where each list contains the ending locations of semiotic spans in input words.
+            do_basic_tokenization: whether to do a pre-processing to separate punctuation marks, recommended to set to True
         """
         self.eval()
 
@@ -148,53 +153,104 @@ class DuplexTaggerModel(NLPModel):
         for ix, sent in enumerate(sents):
             if inst_directions[ix] == constants.INST_BACKWARD:
                 prefix = constants.ITN_PREFIX
-            if inst_directions[ix] == constants.INST_FORWARD:
+            elif inst_directions[ix] == constants.INST_FORWARD:
                 prefix = constants.TN_PREFIX
-            texts.append([prefix] + sent)
+            if do_basic_tokenization:
+                texts.append([prefix] + sent)
+            else:
+                texts.append(prefix + " " + sent)
 
         # Apply the model
+        if do_basic_tokenization:
+            is_split_into_words = True
+        else:
+            is_split_into_words = False
+
         encodings = self._tokenizer(
-            texts, is_split_into_words=True, padding=True, truncation=True, return_tensors='pt'
+            texts, is_split_into_words=is_split_into_words, padding=True, truncation=True, return_tensors='pt'
         )
-        logits = self.model(**encodings.to(self.device)).logits
+
+        inputs = encodings
+        encodings_reduced = None
+
+        # check that the length of the 'input_ids' equals as least the length of the original input
+        # if an input symbol is missing in the tokenizer's vocabulary (such as emoji or a Chinese character), it could be skipped
+        if do_basic_tokenization:
+            len_texts = [len(x) for x in texts]
+        else:
+            len_texts = [len(x.split()) for x in texts]
+        len_ids = [
+            len(self._tokenizer.convert_ids_to_tokens(x, skip_special_tokens=True)) for x in encodings['input_ids']
+        ]
+        idx_valid = [i for i, (t, enc) in enumerate(zip(len_texts, len_ids)) if enc >= t]
+
+        if len(idx_valid) != len(texts):
+            logging.warning(
+                'Some of the examples have symbols that were skipped during the tokenization. Such examples will be skipped.'
+            )
+            for i in range(len(texts)):
+                if i not in idx_valid:
+                    logging.warning(f'Invalid input: {texts[i]}')
+            # skip these sentences and fall back to the input
+            # exclude invalid examples from the encodings
+            encodings_reduced = {k: tensor[idx_valid, :] for k, tensor in encodings.items()}
+            for k, tensor in encodings_reduced.items():
+                if tensor.ndim == 1:
+                    encodings_reduced[k] = tensor.unsqueeze(dim=0)
+            inputs = BatchEncoding(data=encodings_reduced)
+
+        # skip the batch if no valid inputs are present
+        if encodings_reduced and encodings_reduced['input_ids'].numel() == 0:
+            # -1 to exclude tag for the prompt token
+            all_tag_preds = [[constants.SAME_TAG] * (len(x) - 1) for x in texts]
+            nb_spans = [0] * len(texts)
+            span_starts = [] * len(texts)
+            span_ends = [] * len(texts)
+            return all_tag_preds, nb_spans, span_starts, span_ends
+
+        logits = self.model(**inputs.to(self.device)).logits
         pred_indexes = torch.argmax(logits, dim=-1).tolist()
 
         # Extract all_tag_preds for words
         all_tag_preds = []
         batch_size, max_len = encodings['input_ids'].size()
+        pred_idx = 0
         for ix in range(batch_size):
-            raw_tag_preds = [
-                constants.ALL_TAG_LABELS[p] for p in pred_indexes[ix][2:]
-            ]  # remove first special token and task prefix token
-            tag_preds, previous_word_idx = [], None
-            word_ids = encodings.word_ids(batch_index=ix)[2:]
-            for jx, word_idx in enumerate(word_ids):
-                if word_idx is None:
-                    continue
-                if word_idx != previous_word_idx:
-                    tag_preds.append(raw_tag_preds[jx])  # without special token at index 0
-                previous_word_idx = word_idx
+            if ix in idx_valid:
+                # remove first special token and task prefix token
+                raw_tag_preds = [constants.ALL_TAG_LABELS[p] for p in pred_indexes[pred_idx][2:]]
+                tag_preds, previous_word_idx = [], None
+                word_ids = encodings.word_ids(batch_index=ix)[2:]
+                for jx, word_idx in enumerate(word_ids):
+                    if word_idx is None:
+                        continue
+                    if word_idx != previous_word_idx:
+                        tag_preds.append(raw_tag_preds[jx])  # without special token at index 0
+                    previous_word_idx = word_idx
+                pred_idx += 1
+            else:
+                # for excluded examples, use SAME tags for all words
+                tag_preds = [constants.SAME_TAG] * (len(texts[ix]) - 1)
             all_tag_preds.append(tag_preds)
 
-        # Postprocessing
+        # Post-correction of simple tagger mistakes, i.e. I- tag is proceeding the B- tag in a span
         all_tag_preds = [
-            self.postprocess_tag_preds(words, inst_dir, ps)
+            self._postprocess_tag_preds(words, inst_dir, ps)
             for words, inst_dir, ps in zip(sents, inst_directions, all_tag_preds)
         ]
 
         # Decoding
         nb_spans, span_starts, span_ends = self.decode_tag_preds(all_tag_preds)
-
         return all_tag_preds, nb_spans, span_starts, span_ends
 
-    def postprocess_tag_preds(self, words, inst_dir, preds):
+    def _postprocess_tag_preds(self, words: List[str], inst_dir: str, preds: List[str]):
         """ Function for postprocessing the raw tag predictions of the model. It
         corrects obvious mistakes in the tag predictions such as a TRANSFORM span
         starts with I_TRANSFORM_TAG (instead of B_TRANSFORM_TAG).
 
         Args:
             words: The words in the input sentence
-            inst_dir: The direction of the instance (i.e., INST_BACKWARD or INST_FORWARD).
+            inst_dir: The direction of the instance (i.e., constants.INST_BACKWARD or INST_FORWARD).
             preds: The raw tag predictions
 
         Returns: The processed raw tag predictions
@@ -219,17 +275,17 @@ class DuplexTaggerModel(NLPModel):
             final_preds.append(p)
         return final_preds
 
-    def decode_tag_preds(self, tag_preds):
+    def decode_tag_preds(self, tag_preds: List[List[str]]):
         """ Decoding the raw tag predictions to locate the semiotic spans in the
         input texts.
 
         Args:
-            tag_preds: A list of list where each list contains the raw tag predictions for the corresponding input.
+            tag_preds: A list of list where each list contains the raw tag predictions for the corresponding input words.
 
         Returns:
             nb_spans: A list of ints where each int indicates the number of semiotic spans in each input.
-            span_starts: A list of lists where each list contains the starting locations of semiotic spans in an input.
-            span_ends: A list of lists where each list contains the ending locations of semiotic spans in an input.
+            span_starts: A list of lists where each list contains the starting locations of semiotic spans in an input words.
+            span_ends: A list of lists where each list contains the inclusive ending locations of semiotic spans in an input words.
         """
         nb_spans, span_starts, span_ends = [], [], []
         for i, preds in enumerate(tag_preds):
@@ -247,7 +303,6 @@ class DuplexTaggerModel(NLPModel):
             nb_spans.append(cur_nb_spans)
             span_starts.append(cur_span_starts)
             span_ends.append(cur_span_ends)
-
         return nb_spans, span_starts, span_ends
 
     # Functions for processing data
@@ -258,7 +313,7 @@ class DuplexTaggerModel(NLPModel):
             )
             self._train_dl = None
             return
-        self._train_dl = self._setup_dataloader_from_config(cfg=train_data_config, mode="train")
+        self._train_dl = self._setup_dataloader_from_config(cfg=train_data_config, data_split="train")
 
     def setup_validation_data(self, val_data_config: Optional[DictConfig]):
         if not val_data_config or not val_data_config.data_path:
@@ -267,7 +322,7 @@ class DuplexTaggerModel(NLPModel):
             )
             self._validation_dl = None
             return
-        self._validation_dl = self._setup_dataloader_from_config(cfg=val_data_config, mode="val")
+        self._validation_dl = self._setup_dataloader_from_config(cfg=val_data_config, data_split="val")
 
     def setup_test_data(self, test_data_config: Optional[DictConfig]):
         if not test_data_config or test_data_config.data_path is None:
@@ -276,23 +331,24 @@ class DuplexTaggerModel(NLPModel):
             )
             self._test_dl = None
             return
-        self._test_dl = self._setup_dataloader_from_config(cfg=test_data_config, mode="test")
+        self._test_dl = self._setup_dataloader_from_config(cfg=test_data_config, data_split="test")
 
-    def _setup_dataloader_from_config(self, cfg: DictConfig, mode: str):
+    def _setup_dataloader_from_config(self, cfg: DictConfig, data_split: str):
         start_time = perf_counter()
-        logging.info(f'Creating {mode} dataset')
+        logging.info(f'Creating {data_split} dataset')
         input_file = cfg.data_path
         tagger_data_augmentation = cfg.get('tagger_data_augmentation', False)
         dataset = TextNormalizationTaggerDataset(
-            input_file,
-            self._tokenizer,
-            self.transformer_name,
-            cfg.mode,
-            cfg.do_basic_tokenize,
-            tagger_data_augmentation,
-            cfg.lang,
-            cfg.get('use_cache', False),
-            cfg.get('max_insts', -1),
+            input_file=input_file,
+            tokenizer=self._tokenizer,
+            tokenizer_name=self.transformer_name,
+            mode=self.mode,
+            do_basic_tokenize=cfg.do_basic_tokenize,
+            tagger_data_augmentation=tagger_data_augmentation,
+            lang=self.lang,
+            max_seq_length=self.max_sequence_len,
+            use_cache=cfg.get('use_cache', False),
+            max_insts=cfg.get('max_insts', -1),
         )
         data_collator = DataCollatorForTokenClassification(self._tokenizer)
         dl = torch.utils.data.DataLoader(
