@@ -21,7 +21,7 @@ import torch.nn as nn
 from tqdm import tqdm
 
 from nemo.collections.nlp.data.text_normalization import TextNormalizationTestDataset, constants
-from nemo.collections.nlp.data.text_normalization.utils import basic_tokenize
+from nemo.collections.nlp.data.text_normalization.utils import basic_tokenize, post_process_punct
 from nemo.collections.nlp.models.duplex_text_normalization.utils import get_formatted_string
 from nemo.utils import logging
 from nemo.utils.decorators.experimental import experimental
@@ -147,11 +147,13 @@ class DuplexTextNormalizationModel(nn.Module):
                     cur_output_spans.append(output_spans)
             nb_instances = len(cur_final_preds)
             cur_targets_sent = [" ".join(x) for x in cur_targets]
+
             sent_accuracy = TextNormalizationTestDataset.compute_sent_accuracy(
                 cur_final_preds, cur_targets_sent, cur_dirs, self.lang
             )
+
             class_accuracy = TextNormalizationTestDataset.compute_class_accuracy(
-                self.input_preprocessing(list(cur_inputs)),
+                [basic_tokenize(x, lang=self.lang) for x in cur_inputs],
                 cur_targets,
                 cur_tag_preds,
                 cur_dirs,
@@ -180,8 +182,8 @@ class DuplexTextNormalizationModel(nn.Module):
                 "class_accuracy": log_class_accuracies,
             }
             # Write errors to log file
-            for _input, tag_pred, final_pred, target in zip(
-                cur_inputs, cur_tag_preds, cur_final_preds, cur_targets_sent
+            for _input, tag_pred, final_pred, target, classes in zip(
+                cur_inputs, cur_tag_preds, cur_final_preds, cur_targets_sent, cur_classes
             ):
                 if not TextNormalizationTestDataset.is_same(final_pred, target, direction, self.lang):
                     if direction == constants.INST_BACKWARD:
@@ -192,9 +194,11 @@ class DuplexTextNormalizationModel(nn.Module):
                         tn_error_ctx += 1
                     formatted_input_str = get_formatted_string(basic_tokenize(_input, lang=self.lang))
                     formatted_tag_pred_str = get_formatted_string(tag_pred)
+                    class_str = " ".join(classes)
                     error_f.write(f'Original Input : {_input}\n')
                     error_f.write(f'Input          : {formatted_input_str}\n')
                     error_f.write(f'Predicted Tags : {formatted_tag_pred_str}\n')
+                    error_f.write(f'Ground Classes : {class_str}\n')
                     error_f.write(f'Predicted Str  : {final_pred}\n')
                     error_f.write(f'Ground-Truth   : {target}\n')
                     error_f.write('\n')
@@ -209,74 +213,65 @@ class DuplexTextNormalizationModel(nn.Module):
 
         # Close log file
         error_f.close()
-
+        logging.info(f'Errors are saved at {errors_log_fp}.')
         return results
 
     # Functions for inference
-    def _infer(self, sents: List[str], inst_directions: List[str]):
-        """ Main function for Inference
+    def _infer(self, sents: List[str], inst_directions: List[str], do_basic_tokenization=True):
+        """
+        Main function for Inference
+
+        If the 'joint' mode is used, "sents" will include both spoken and written forms on each input sentence,
+        and "inst_directions" will include both constants.INST_BACKWARD and constants.INST_FORWARD
+
         Args:
             sents: A list of input texts.
-            inst_directions: A list of str where each str indicates the direction of the corresponding instance (i.e., INST_BACKWARD for ITN or INST_FORWARD for TN).
+            inst_directions: A list of str where each str indicates the direction of the corresponding instance \
+                (i.e., constants.INST_BACKWARD for ITN or constants.INST_FORWARD for TN).
+            do_basic_tokenization: whether to do a pre-processing to separate punctuation marks,
+                recommended to set to True
 
         Returns:
             tag_preds: A list of lists where the inner list contains the tag predictions from the tagger for each word in the input text.
             output_spans: A list of lists where each list contains the decoded semiotic spans from the decoder for an input text.
             final_outputs: A list of str where each str is the final output text for an input text.
         """
-        # Preprocessing
-        sents = self.input_preprocessing(list(sents))
+        original_sents = [s for s in sents]
+        # Separate into words
+        if do_basic_tokenization:
+            sents = [self.decoder.processor.tokenize(x).split() for x in sents]
 
         # Tagging
-        tag_preds, nb_spans, span_starts, span_ends = self.tagger._infer(sents, inst_directions)
+        # span_ends included, returns index wrt to words in input without auxiliary words
+        tag_preds, nb_spans, span_starts, span_ends = self.tagger._infer(
+            sents, inst_directions, do_basic_tokenization=do_basic_tokenization
+        )
         output_spans = self.decoder._infer(sents, nb_spans, span_starts, span_ends, inst_directions)
-        # Preprare final outputs
+
+        if not do_basic_tokenization:
+            sents = [x.split() for x in sents]
+
+        # Prepare final outputs
         final_outputs = []
         for ix, (sent, tags) in enumerate(zip(sents, tag_preds)):
-            cur_words, jx, span_idx = [], 0, 0
-            cur_spans = output_spans[ix]
-            while jx < len(sent):
-                tag, word = tags[jx], sent[jx]
-                if constants.SAME_TAG in tag:
-                    cur_words.append(word)
-                    jx += 1
-                elif constants.PUNCT_TAG in tag:
-                    jx += 1
-                else:
-                    jx += 1
-                    cur_words.append(cur_spans[span_idx])
-                    span_idx += 1
-                    while jx < len(sent) and tags[jx] == constants.I_PREFIX + constants.TRANSFORM_TAG:
+            try:
+                cur_words, jx, span_idx = [], 0, 0
+                cur_spans = output_spans[ix]
+                while jx < len(sent):
+                    tag, word = tags[jx], sent[jx]
+                    if constants.SAME_TAG in tag:
+                        cur_words.append(word)
                         jx += 1
-            cur_output_str = ' '.join(cur_words)
-            cur_output_str = ' '.join(basic_tokenize(cur_output_str, self.lang))
-            final_outputs.append(cur_output_str)
+                    else:
+                        jx += 1
+                        cur_words.append(cur_spans[span_idx])
+                        span_idx += 1
+                        while jx < len(sent) and tags[jx] == constants.I_PREFIX + constants.TRANSFORM_TAG:
+                            jx += 1
+                cur_output_str = self.decoder.processor.detokenize(cur_words)
+                cur_output_str = post_process_punct(input=original_sents[ix], nn_output=cur_output_str)
+                final_outputs.append(cur_output_str)
+            except IndexError:
+                logging.warning(f"Input sent is too long and will be skipped - {original_sents[ix]}")
+                final_outputs.append(original_sents[ix])
         return tag_preds, output_spans, final_outputs
-
-    def input_preprocessing(self, sents):
-        """ Function for preprocessing the input texts. The function first does
-        some basic tokenization. For English, it then also processes Greek letters
-        such as Δ or λ (if any).
-
-        Args:
-            sents: A list of input texts.
-
-        Returns: A list of preprocessed input texts.
-        """
-        # Basic Preprocessing and Tokenization
-        if self.lang == constants.ENGLISH:
-            for ix, sent in enumerate(sents):
-                sents[ix] = sents[ix].replace('+', ' plus ')
-                sents[ix] = sents[ix].replace('=', ' equals ')
-                sents[ix] = sents[ix].replace('@', ' at ')
-                sents[ix] = sents[ix].replace('*', ' times ')
-        sents = [basic_tokenize(sent, self.lang) for sent in sents]
-
-        # Greek letters processing
-        if self.lang == constants.ENGLISH:
-            for ix, sent in enumerate(sents):
-                for jx, tok in enumerate(sent):
-                    if tok in constants.EN_GREEK_TO_SPOKEN:
-                        sents[ix][jx] = constants.EN_GREEK_TO_SPOKEN[tok]
-
-        return sents
