@@ -44,7 +44,7 @@
 
 import torch
 
-from nemo.collections.tts.helpers.helpers import binarize_attention_parallel
+from nemo.collections.tts.helpers.helpers import binarize_attention_parallel, regulate_len
 from nemo.core.classes import NeuralModule, typecheck
 from nemo.core.neural_types.elements import (
     EncodedRepresentation,
@@ -61,28 +61,6 @@ from nemo.core.neural_types.elements import (
 from nemo.core.neural_types.neural_type import NeuralType
 
 
-def regulate_len(durations, enc_out, pace=1.0, mel_max_len=None):
-    """If target=None, then predicted durations are applied"""
-    dtype = enc_out.dtype
-    reps = durations.float() / pace
-    reps = (reps + 0.5).long()
-    dec_lens = reps.sum(dim=1)
-
-    max_len = dec_lens.max()
-    reps_cumsum = torch.cumsum(torch.nn.functional.pad(reps, (1, 0, 0, 0), value=0.0), dim=1)[:, None, :]
-    reps_cumsum = reps_cumsum.to(dtype)
-
-    range_ = torch.arange(max_len).to(enc_out.device)[None, :, None]
-    mult = (reps_cumsum[:, :, :-1] <= range_) & (reps_cumsum[:, :, 1:] > range_)
-    mult = mult.to(dtype)
-    enc_rep = torch.matmul(mult, enc_out)
-
-    if mel_max_len:
-        enc_rep = enc_rep[:, :mel_max_len]
-        dec_lens = torch.clamp_max(dec_lens, mel_max_len)
-    return enc_rep, dec_lens
-
-
 def average_pitch(pitch, durs):
     durs_cums_ends = torch.cumsum(durs, dim=1).long()
     durs_cums_starts = torch.nn.functional.pad(durs_cums_ends[:, :-1], (1, 0))
