@@ -24,17 +24,28 @@ from functools import total_ordering
 from pathlib import Path
 from typing import Dict, List, Optional, Union
 
-import hydra
 import wrapt
-from omegaconf import DictConfig, OmegaConf
 
 import nemo
-from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
 from nemo.core.neural_types import NeuralType, NeuralTypeComparisonResult
 from nemo.utils import logging
 from nemo.utils.cloud import maybe_download_from_cloud
 from nemo.utils.model_utils import import_class_by_path, maybe_update_config_version
 
+# TODO @blisc: Perhaps refactor instead of import guarding
+_HAS_HYDRA = True
+try:
+    import hydra
+    from omegaconf import DictConfig, OmegaConf
+    from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
+except ModuleNotFoundError:
+    _HAS_HYDRA = False
+    from nemo.utils.exceptions import CheckInstall
+
+    class SaveRestoreConnector(CheckInstall):
+        pass
+
+
 __all__ = ['Typing', 'FileIO', 'Model', 'Serialization', 'typecheck']
 
 
@@ -418,30 +429,32 @@ class Typing(ABC):
 
 class Serialization(ABC):
     @classmethod
-    def from_config_dict(cls, config: DictConfig):
+    def from_config_dict(cls, config: 'DictConfig'):
         """Instantiates object using DictConfig-based configuration"""
         # Resolve the config dict
-        if isinstance(config, DictConfig):
-            config = OmegaConf.to_container(config, resolve=True)
-            config = OmegaConf.create(config)
-            OmegaConf.set_struct(config, True)
+        if _HAS_HYDRA:
+            if isinstance(config, DictConfig):
+                config = OmegaConf.to_container(config, resolve=True)
+                config = OmegaConf.create(config)
+                OmegaConf.set_struct(config, True)
 
-        config = maybe_update_config_version(config)
+            config = maybe_update_config_version(config)
 
         # Hydra 0.x API
-        if ('cls' in config or 'target' in config) and 'params' in config:
+        if ('cls' in config or 'target' in config) and 'params' in config and _HAS_HYDRA:
             # regular hydra-based instantiation
             instance = hydra.utils.instantiate(config=config)
         # Hydra 1.x API
-        elif '_target_' in config:
+        elif '_target_' in config and _HAS_HYDRA:
             # regular hydra-based instantiation
             instance = hydra.utils.instantiate(config=config)
         else:
             instance = None
             imported_cls_tb = None
+            instance_init_error = None
             # Attempt class path resolution from config `target` class (if it exists)
             if 'target' in config:
-                target_cls = config.target
+                target_cls = config["target"]  # No guarantee that this is a omegaconf class
                 imported_cls = None
                 try:
                     # try to import the target class
@@ -458,8 +471,9 @@ class Serialization(ABC):
 
                     try:
                         instance = imported_cls(cfg=config)
-                    except Exception:
+                    except Exception as e:
                         imported_cls_tb = traceback.format_exc()
+                        instance_init_error = str(e)
                         instance = None
 
             # target class resolution was unsuccessful, fall back to current `cls`
@@ -470,21 +484,28 @@ class Serialization(ABC):
                         f"Falling back to `cls`.\n"
                         f"{imported_cls_tb}"
                     )
-                instance = cls(cfg=config)
+                try:
+                    instance = cls(cfg=config)
+                except Exception as e:
+                    if imported_cls_tb is not None:
+                        logging.error(f"Instance failed restore_from due to: {instance_init_error}")
+                        logging.error(f"{imported_cls_tb}")
+                    raise e
 
         if not hasattr(instance, '_cfg'):
             instance._cfg = config
         return instance
 
-    def to_config_dict(self) -> DictConfig:
+    def to_config_dict(self) -> 'DictConfig':
         """Returns object's configuration to config dictionary"""
-        if hasattr(self, '_cfg') and self._cfg is not None and isinstance(self._cfg, DictConfig):
+        if hasattr(self, '_cfg') and self._cfg is not None:
             # Resolve the config dict
-            config = OmegaConf.to_container(self._cfg, resolve=True)
-            config = OmegaConf.create(config)
-            OmegaConf.set_struct(config, True)
+            if _HAS_HYDRA and isinstance(self._cfg, DictConfig):
+                config = OmegaConf.to_container(self._cfg, resolve=True)
+                config = OmegaConf.create(config)
+                OmegaConf.set_struct(config, True)
 
-            config = maybe_update_config_version(config)
+                config = maybe_update_config_version(config)
 
             self._cfg = config
 
