@@ -56,7 +56,7 @@ def write_file(file_path: str, data: List[str]):
 def parse_args():
     parser = ArgumentParser()
     parser.add_argument("--input", help="input file path", required=True, type=str)
-    parser.add_argument("--language", help="language", choices=['en'], default="en", type=str)
+    parser.add_argument("--language", help="language", choices=['en', 'de'], default="en", type=str)
     parser.add_argument("--output", help="output file path", required=True, type=str)
     parser.add_argument("--verbose", help="print denormalization info. For debugging", action='store_true')
     return parser.parse_args()
